#!/usr/bin/env python
"""
to compute the confuse matrix and auc
inputfile: >probability; number; 2011-09-21 add the return(ACC,Sn,Sp,MCC,Precsion,F-measure) in main
"""
import os, math, sys


def roc_xy_axis(labels, results, cutoff, P_label, N_label):
    #compute the roc curve
    (tp, fp) = (0, 0)
    (fn, tn) = (0, 0)
    # count the number
    for i in range(len(results)):
        #store the probability
        xpos = results[i]
        if labels[i] == P_label and float(xpos) > cutoff:
            tp = tp + 1
        elif labels[i] == P_label and float(xpos) <= cutoff:
            fn = fn + 1
        elif labels[i] == N_label and float(xpos) > cutoff:
            fp = fp + 1
        elif labels[i] == N_label and float(xpos) <= cutoff:
            tn = tn + 1
    p = tp + fn
    n = fp + tn
    #ACC
    if p + n > 0:
        ACC = (tp + tn) * 1.0 / (p + n)
    else:
        ACC = 0.0
        print("Accuracy divided zero, cutoff: %f" % cutoff)
    #Sn
    if (p > 0):
        Sn = tp * 1.0 / p
    else:
        Sn = 0.0
        print("Sn/TPR divided zero, cutoff: %f" % cutoff)
    #Sp
    if (n > 0):
        Sp = tn * 1.0 / n
    else:
        Sp = 0.0
        print("Sn/TNR divided zero, cutoff: %f" % cutoff)
    #MCC
    MCCfenzi = tp * tn - fp * fn
    MCCfenmu = math.sqrt(p * n * (tp + fp) * (fn + tn))
    if MCCfenmu > 0:
        MCC = MCCfenzi * 1.0 / MCCfenmu
    else:
        MCC = 0.0
        print("MCC divided zero, cutoff: %f" % cutoff)
    #Precision
    if tp + fp > 0:
        Precision = tp * 1.0 / (tp + fp)
    else:
        Precision = 0.0
        print("Precision divided zero, cutoff: %f" % cutoff)
    #Recall &
    Recall = Sn
    if Precision + Recall > 0:
        F_measure = 2.0 * (Precision * Recall) / (Precision + Recall)
    else:
        F_measure = 0.0
        print("F_measure divided zero, cutoff: %f" % cutoff)
    if tp + fp > 0:  #Precision/positive predictive value
        ppv = tp * 1.0 / (tp + fp)
    else:
        ppv = 0.0
        print("PPV divided zero, cutoff: %f" % cutoff)

    if tn + fn > 0:  #Precision/positive predictive value
        npv = tn * 1.0 / (tn + fn)
    else:
        npv = 0.0
        print("NPV divided zero, cutoff: %f" % cutoff)

    return (tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision, F_measure, ppv,
            npv)
    #return (1-Sp,Sn)


def compute_auc(xyaxis):
    AUC = 0.0
    prev_x = 0.0
    prev_y = 0.0
    xyaxis.sort()
    for i in range(0, len(xyaxis)):
        x, y = xyaxis[i]
        AUC += (x - prev_x) * (y + prev_y) * 0.5
        prev_x = x
        prev_y = y

    last_x, last_y = xyaxis[-1]

    AUC += (1 - last_x) * (1 + last_y) * 0.5
    return AUC


###
def main(fr_n, P_label, N_label):
    # store the results
    labels = []  #actual labels
    results = []  #probablities
    fr = open(fr_n, 'r')
    lines = [line.strip() for line in fr.readlines()]
    fr.close()  # v1 id; v2 probability v3 actuals
    for i in range(len(lines)):
        if len(lines[i]) > 0:  # and lines[i][0]=='>':
            #proid=lines[i]
            predict = lines[i].split(',')
            labels.append(predict[0])
            results.append(float(predict[1]))
            #actual=lines[i+2]
            #for j in range(len(predict)-1):  #remove the end ","
            #read in probability
            #	results.append(float(predict[j]))
            #for j in range(len(actual)):
            #read in labels
            #if actual[j]=='*':
            #	labels.append(P_label)
            #else:
            #	labels.append(N_label)
    fw_n = fr_n + ".auc.confuse.matrix.txt"
    fw = open(fw_n, 'w')
    fw.write("#labels predictions\n")
    if len(results) != len(labels):
        print("%s results and labels length is different")
    for i in range(len(results)):
        fw.write("%s %9f\n" % (labels[i], results[i]))

    xyaxis = []
    cutoff = 0.0
    while (cutoff <= 1.001):
        #sp1,sn=roc_xy_axis(labels,results,cutoff,P_label,N_label)
        (tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision,
         F_measure) = roc_xy_axis(labels, results, cutoff, P_label, N_label)
        xyaxis.append((1 - Sp, Sn))
        #print cutoff  #2010-09-01
        cutoff = cutoff + 0.02
    #print xaxis
    xyaxis.sort()
    #print xyaxis #2010-09-01
    fw.write("==ROC x-axis:1-Sp y-axis:Sn==\n")
    for k in range(len(xyaxis)):
        xa, yb = xyaxis[k]
        fw.write("%f   %f\n" % (xa, yb))
    myauc = compute_auc(xyaxis)
    print "AUC= ", myauc
    fw.write("AUC=%.3f\n" % myauc)
    #compute the confuse matrix
    cutoff = 0.5
    (tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision,
     F_measure) = roc_xy_axis(labels, results, cutoff, P_label, N_label)
    #write the confuse matrix
    fw.write("------Confusion  Matrix-----\n")
    fw.write("TP FP|%5d %5d\n" % (tp, fp))
    fw.write("FN TN|%5d %5d\n" % (fn, tn))
    fw.write("Accuracy            %.3f \n" % ACC)
    fw.write("Sn/TPR/recall       %.3f \n" % Sn)
    fw.write("Sp/TNR              %.3f \n" % Sp)
    fw.write("MCC                 %.3f \n" % MCC)
    fw.write("Precision/PPR       %.3f \n" % Precision)
    fw.write("F_measure           %.3f \n" % F_measure)
    fw.close()
    return (ACC, Sn, Sp, MCC, Precision, F_measure)


if __name__ == "__main__":
    #comput_svm(float(sys.argv[1]),float(sys.argv[2]))
    if len(sys.argv) < 3:
        print("Usage: my_auc_confuse.py  fr_n P_label N_label")
    else:
        fr_n = sys.argv[1]
        P_label = sys.argv[2]
        N_label = sys.argv[3]
        main(fr_n, P_label, N_label)
