# coding:utf-8
import numpy as np
from math import factorial


def Q(args):
    N = len(args)
    Vs = [1]
    for k in range(1, N + 1):
        v = 0.
        for i in range(1, k + 1):
            part = (-1)**(i - 1) * Vs[k - i] * (
                args[k - i]**i) * 1.0 / factorial(i)
            v += part
        Vs.append(v)
    res = Vs[N]
    res *= factorial(N)
    return res


def run():
    res = []
    with open("./myranks1.csv") as f:
        for line in f:
            if line[0] != "#":
                tmp = line.strip().split(",")
                #print "{}".format(tmp[0])
                scores = np.sort([float(i) for i in tmp[2:]])
                res.append("{},{},{}\n".format(tmp[0], tmp[1], Q(scores)))

    with open("./result1.csv", 'w') as f:
        for re in res:
            f.write(re)
    return (0)


def get_order_predictions(newXX, YY, wfnFlag):
    ### Q(r1,r2,...,rN)=N!V_N
    labels = YY
    if len(newXX.shape) == 1:
        return (newXX)
    else:
        row, col = newXX.shape
    predicts = []
    for i in range(row):
        aRowRanks = np.sort(newXX[i, :])
        aQscore = Q(aRowRanks)
        predicts.append(aQscore)

    if wfnFlag:
        wfnFlag = "check.best.predicts.csv"
        fw = open(wfnFlag, 'w')
        for k in range(row):
            fw.write("%s,%f\n" % (labels[k], predicts[k]))
        fw.close()
    return (predicts)


def get_predictions(newXX, YY, wfnFlag, mFlag, Pnum):
    #XX features; YY: label; wfnFlag write file; mFlag: min/max/avg/order
    predicts = []
    if mFlag == "order":
        predicts = get_order_predictions(newXX, YY, wfnFlag)
    elif mFlag == "avg":
        predicts = np.mean(newXX, axis=1)  #0 col  1 row
    elif mFlag == "min":
        predicts = np.min(newXX, axis=1)
    elif mFlag == "max":
        predicts = np.max(newXX, axis=1)
    elif mFlag == "vote":
        sidx = np.argsort(-newXX, axis=0)  # col descreasing
        d1Flag = False  # 1d array or 2-d array
        row = np.shape(sidx)[0]
        try:
            col = np.shape(sidx)[1]  #row
        except IndexError:
            print("newXX is list 1d array")
            d1Flag = True
        if d1Flag:
            for k in range(row):
                if k in sidx[0:Pnum]:
                    predicts.append(1)
                else:
                    predicts.append(0)
        else:
            # 2d array
            if col % 2 == 0:
                half_size = col / 2
            else:
                half_size = (col / 2) + 1  #5/2=2 ; over half size
            for k in range(row):
                counter = 0
                for j in range(col):
                    if k in sidx[:, j][0:Pnum]:
                        counter += 1
                if counter >= half_size:
                    predicts.append(1)
                else:
                    predicts.append(0)

    return (predicts)


from scipy import stats


def get_pvalues(predicts, N):
    ### get the p-value using gamma distribution N>5,beta distribution N<=5;
    L = len(predicts)
    if N > 5:
        m1 = np.average(predicts)  #m^bar
        m2 = np.sum((predicts - m1)**2) * 1.0 / L  #  S^2 1/N SUM(X-mean(X))^2
        m3 = np.sum((predicts - m1)**3) * 1.0 / L  #  m3
        a_shape = 4.0 * (m2) * (m2) * (m2) / (m3 * m3)
        b_scale = m3 * 1.0 / (2.0 * m2)
        c_loc = m1 - 2.0 * (m2) * (m2) / m3
        p_values = 1 - stats.gamma.cdf(predicts, a_shape, c_loc,
                                       b_scale)  #cdf(x, a, loc=0, scale=1)
        #fit_alpha,fit_loc,fit_beta=stats.gamma.fit(predicts)
        #p_values=1-stats.gamma.cdf(predicts,fit_alpha,fit_loc,fit_beta)

    else:  #beta distribution
        m1 = np.average(predicts)
        m2sample = np.sum((predicts - m1)**2) * 1.0 / (L - 1)
        aa = m1 * (m1 * (1 - m1) * 1.0 / m2sample - 1)
        bb = (1 - m1) * (m1 * (1 - m1) * 1.0 / m2sample - 1)
        #fit_a,fit_b,fit_loc,fit_beta=stats.beta.fit(predicts)
        #print(fit_a,fit_b,fit_loc,fit_beta)
        #p_values=1-stats.beta.cdf(predicts,fit_a,fit_b,fit_loc,fit_beta)
        p_values = 1 - stats.beta.cdf(predicts, aa, bb)
    return (p_values)


#mydat=np.loadtxt("check.best.predicts.csv",delimiter=",",dtype=float)
#N=2
#predicts=mydat[:,1]
#print(get_pvalues(predicts,N))
#if __name__ == '__main__':

#run()
#print Q([1,2])
