# -*- coding: utf-8 -*-
"""
Created on Tue May 16 10:05:35 2017

@author: Cliff
"""
#import deep
import numpy as np
from rank_features import rank_features
from scale import scale_model
### evaluate the model
from sklearn.linear_model import LogisticRegression
import math
EPS = 0.00001
from my_auc_confuse_col import roc_xy_axis


def counter_confuse_matrix_4nx(labels, results, Pnum, P_label, N_label,
                               maxFlag):
    #count the top100/top200/..../top600 of essential proteins
    # top Pnum is predicted as positive
    (tp, fp) = (0, 0)
    (fn, tn) = (0, 0)
    # count the number

    if maxFlag:
        sort_idxes = np.argsort(results)[::-1]  #decrease
    else:
        sort_idxes = np.argsort(results)  #increase , less better
    #### top

    for xi in range(Pnum):
        #top Pnum is predicte as positive
        i = sort_idxes[xi]
        if labels[i] == P_label:
            tp = tp + 1
        elif labels[i] == N_label:
            fp = fp + 1
    for xi in range(Pnum, len(results)):
        i = sort_idxes[xi]
        # other prediction as negative
        if labels[i] == P_label:
            fn = fn + 1
        elif labels[i] == N_label:
            tn = tn + 1
    p = tp + fn
    n = fp + tn
    #ACC
    if p + n > 0:
        ACC = (tp + tn) * 1.0 / (p + n)
    else:
        ACC = 0.0
        print("Accuracy divided zero")
    #Sn
    if (p > 0):
        Sn = tp * 1.0 / p
    else:
        Sn = 0.0
        print("Sn/TPR divided zero")
    #Sp
    if (n > 0):
        Sp = tn * 1.0 / n
    else:
        Sp = 0.0
        print("Sn/TNR divided zero")
    #MCC
    MCCfenzi = tp * tn - fp * fn
    MCCfenmu = math.sqrt(p * n * (tp + fp) * (fn + tn))
    if MCCfenmu > 0:
        MCC = MCCfenzi * 1.0 / MCCfenmu
    else:
        MCC = 0.0
        print("MCC divided zero")
    #Precision
    if tp + fp > 0:
        Precision = tp * 1.0 / (tp + fp)
    else:
        Precision = 0.0
        print("Precision divided zero")
    #Recall &
    Recall = Sn
    if Precision + Recall > 0:
        F_measure = 2.0 * (Precision * Recall) / (Precision + Recall)
    else:
        F_measure = 0.0
        print("F_measure divided zero")
    if tp + fp > 0:  #Precision/positive predictive value
        ppv = tp * 1.0 / (tp + fp)
    else:
        ppv = 0.0
        print("PPV divided zero")

    if tn + fn > 0:  #Precision/positive predictive value
        npv = tn * 1.0 / (tn + fn)
    else:
        npv = 0.0
        print("NPV divided zero")

    return (tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision, F_measure, ppv,
            npv)


    #return (1-Sp,Sn)
def get_predictions_meta(XX, YY, wfnFlag):
    labels = YY
    results = np.average(XX, axis=1)
    cutoff = 0.5
    P_label = 1
    N_label = 0
    tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision, F_measure = roc_xy_axis(
        labels, results, cutoff, P_label, N_label)
    return (ACC)


def get_predictions(XX, YY, wfnFlag):
    ### get the scale dat
    fwn = "check.csv"
    fw = open(fwn, 'w')
    for item in YY:
        fw.write("%d\n" % item)
    fw.close()
    mylr = LogisticRegression().fit(XX, YY)
    yhat = mylr.predict_proba(XX)  # prob 0/1
    ### compute acc
    #counter=0
    labels = YY
    results = yhat[:, 1]
    cutoff = 0.5
    P_label = 1
    N_label = 0
    tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision, F_measure = roc_xy_axis(
        labels, results, cutoff, P_label, N_label)
    if wfnFlag:
        predf = "best.predicts.csv"
        fw = open(predf, 'w')

        for k in range(len(YY)):
            fw.write("%d,%f\n" % (YY[k], yhat[k][1]))
            #print(k,YY[k],yhat[k])
        fw.close()  #print(yhat)
    return (MCC)


def evaluate_predictions(predicts, labels, wfnFlag, cutoff=0.5):
    #counter=0
    #cutoff=0.5
    P_label = 1
    N_label = 0
    tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision, F_measure, ppv, npv = roc_xy_axis(
        labels, predicts, cutoff, P_label, N_label)
    if wfnFlag:
        predf = "best.predicts.csv"
        fw = open(predf, 'w')

        for k in range(len(labels)):
            fw.write("%d,%f\n" % (labels[k], predicts[k]))
            #print(k,YY[k],yhat[k])
        fw.close()  #print(yhat)
    mymeasures = [
        tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision, F_measure, ppv, npv
    ]
    return (mymeasures)


def evaluate_predictions_4nx(predicts, labels, wfnFlag, Pnum, maxFlag=True):
    #Pnum: top Pnum as the predicted positive samples
    #cutoff=0.5
    P_label = 1
    N_label = 0
    #counter_confuse_matrix_4nx(labels,results,Pnum,P_label,N_label,maxFlag)
    tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision, F_measure, ppv, npv = counter_confuse_matrix_4nx(
        labels, predicts, Pnum, P_label, N_label, maxFlag)
    if wfnFlag:
        predf = "best.predicts.csv"
        fw = open(predf, 'w')

        for k in range(len(labels)):
            fw.write("%d,%f\n" % (labels[k], predicts[k]))
            #print(k,YY[k],yhat[k])
        fw.close()  #print(yhat)
    mymeasures = [
        tp, fn, fp, tn, p, n, ACC, Sn, Sp, MCC, Precision, F_measure, ppv, npv
    ]
    return (mymeasures)


def my_read_predictions(outf):
    #outf="predicts.pcc.csv"
    outs = np.loadtxt(outf, delimiter=",", dtype=float)
    row, col = outs.shape

    label = np.array(outs[:, 0], dtype=int)
    print(label)
    scores = outs[:, 1]
    return (scores, label)


def my_rank(scores, label, maxFlag):
    ### output top100/200/.../600; top1%,5%,...25%.
    ### results
    scores = np.array(scores, dtype=float)
    label = np.array(label, dtype=int)
    results = []
    if maxFlag:
        sort_idxes = np.argsort(-scores)  #decrease
    else:
        sort_idxes = np.argsort(scores)  #increase , less better
    #### top
    top1s = [100, 200, 300, 400, 500, 600]
    top2s = [0.01, 0.05, 0.10, 0.15, 0.20, 0.25]
    #### print top-n
    for item in top1s:
        top_idxes = sort_idxes[0:item]
        top_label = list(label[top_idxes])
        mycount = top_label.count(1)
        #print("Top %3d: %3d" %(item,mycount))
        #print(top_label)
        results.append((item, mycount))
    #### print top-%
    #print("##### percent")
    for item in top2s:
        nn = int(np.ceil(item * len(label)))  # ceiling
        #print(type(nn))
        top_idxes = sort_idxes[0:nn]
        top_label = list(label[top_idxes])
        mycount = top_label.count(1)
        #print("Top %.2f %4d %3d" %(item,nn,mycount))
        results.append((nn, mycount))

    return (results)


#myorder=[7,10,14,13,8,12,15,17,18,11,16,9,19,0,3,1,5,6,4,2,21,20]
def get_dat_rm_k(mydat, k, xmyorder):
    mydat = np.array(mydat, dtype=float)
    myorder = [item for item in xmyorder]
    ndat = []
    for i in range(k):
        myorder.pop()
        #print(i,myorder)
    #print("===")
    if len(myorder) > 0:
        ndat = mydat[:, myorder]
    else:
        ndat = []
    return (np.array(ndat))


def get_dat_add_k(mydat, k, xmyorder):
    mydat = np.array(mydat, dtype=float)
    myorder = [item for item in xmyorder]
    ndat = []
    #print("===",myorder[0:k])
    if len(myorder[0:k]) > 0:
        ndat = mydat[:, myorder[0:k]]
    else:
        ndat = []

    return (np.array(ndat))


#==============================================================================
# xx=np.array([[1,2,3],[4,5,6]])
# k=2
# myorder=[0,2,1]
# print(xx)
# print(get_dat_add_k(xx,k,myorder))
# print("**",myorder)
#==============================================================================
#my_rank(outf)
def my_read_dat(frn):
    ### csv, header; 1st: header,label,features
    fr = open(frn, 'r')
    mydat = []
    lines = [line.strip() for line in fr.readlines()]
    fr.close()
    rownames = []
    for k in range(len(lines)):
        if lines[k]:
            if lines[k][0] == '#':
                header = lines[k].split(',')[2:]
            elif lines[k][0] != '#':
                aline = lines[k].split(',')
                bline = []
                ## 1st column is node name
                for j in range(len(aline)):
                    if j == 0:
                        rownames.append(aline[j])
                    elif j == 1:  #label
                        bline.append(float(aline[j]))
                    else:
                        bline.append(float(aline[j]))
                mydat.append(bline)
    return (mydat, header, rownames)


from orders import get_order_predictions, get_pvalues


def out_measures(frn, fwn, PvalFlag):
    ### output top100/200/.../600/1%/...25%...
    #frn="model.features.scale.csv"
    #frn="myranks1.csv" # node;label;features
    mydat, header, rownames = my_read_dat(frn)
    mydat = np.array(mydat)
    YY = [int(item) for item in mydat[:, 0]]  #label
    XX = mydat[:, 1:]  #features

    #print(mydat)
    #fwn="results1_add_pcc.csv"
    fw = open(fwn, 'w')
    fw.write("#method,Top100,Top200,Top300,Top400,Top500,Top600,")
    fw.write("Top1%,Top5%,Top10%,Top15%,Top20%,Top25%,")
    fw.write("tp,fn,fp,tn,p,n,ACC,Sn,Sp,MCC,Precision,F_measure,ppv,npv\n")

    wfnFlag = False  #True  #False
    #acc0=get_predictions(XX,YY,wfnFlag)
    #"""
    print(XX.shape)
    #print(acc0)
    #myorder=[22,19,15,14,18, 17,8,13,16,12,10,0,11,1, 9,5,3,21,4,2,6,20,7]#pcc
    #myorder=[22,19,15,14,18, 17,8,16,0,1,5,3,21,4,2,6,20,7]#pcc
    myorder = [
        19, 15, 14, 18, 17, 8, 13, 16, 12, 10, 0, 11, 1, 9, 5, 3, 21, 4, 2, 6,
        20, 7
    ]  #pcc
    ### save best model
    #myorder=[19,18,14,17,15,7]
    fwn2 = "../best.predicts.csv"
    fw2 = open(fwn2, 'w')
    fw2.write("#name,label,predicts\n")

    for k in range(len(myorder)):
        #newXX=get_dat_rm_k(XX,k,myorder)
        newXX = get_dat_add_k(XX, k + 1, myorder)
        print(k, newXX.shape)
        if k == 2:
            wfnFlag = True
            np.savetxt("check.ss.csv", newXX, fmt="%f", delimiter=",")
        else:
            wfnFlag = False
        if len(newXX) > 0:
            predicts = get_order_predictions(newXX, YY, wfnFlag)
            #acc=get_predictions(newXX,YY,wfnFlag)
            if PvalFlag:  #using p-values
                pvalues = get_pvalues(predicts, k + 1)
                maxFlag = True
            else:  #just used Q
                pvalues = predicts
                maxFlag = False  #less better
            #predicts=np.array(predicts,dtype=float)*(-1)
            YY = np.array(YY, dtype=int)
            #print(predicts)
            ### save best
            if k == 2:
                for xi in range(len(predicts)):
                    fw2.write("%s,%d,%f\n" % (rownames[xi], YY[xi],
                                              predicts[xi]))
                fw2.close()
            out_ranks = my_rank(pvalues, YY,
                                maxFlag)  #top100/200/.../600/top1%...
            if maxFlag:
                scale_pvalues = (pvalues - min(pvalues)) / (
                    max(pvalues) - min(pvalues))
            else:
                scale_pvalues = (pvalues - max(pvalues)) / (
                    min(pvalues) - max(pvalues))
            #if k==2:
            #    wfnFlag=True
            #else:
            #    wfnFlag=False
            confuse = evaluate_predictions(scale_pvalues, YY, wfnFlag)
            for i in range(len(out_ranks)):
                if i == 0:
                    fw.write("%s,%d," % (header[myorder[k]], out_ranks[i][1]))
                else:
                    fw.write("%d," % out_ranks[i][1])
            for i in range(len(confuse)):
                fw.write("%f," % confuse[i])
            fw.write('\n')
    fw.close()

    return (0)


def out_measure_eachfeatures(frn, fwn, MaxBetterFlag=False, nxFlag="ydip"):
    ### output each features' prediction
    ### MaxBetter bigger the value is,  the better resutls
    mydat, header, rownames = my_read_dat(frn)
    mydat = np.array(mydat)
    YY = [int(item) for item in mydat[:, 0]]  #label
    XX = mydat[:, 1:]  #features
    maxFlag = MaxBetterFlag
    wfnFlag = False  #True: write confuse matrix
    if nxFlag == "ydip" or nxFlag == "ydip4746":
        Pnum = 1130
    elif nxFlag == "ybiogrid":
        Pnum = 1199
    elif nxFlag == "ydip5093":
        Pnum = 1167
    else:
        Pnum = int(np.shape(XX)[0] * 0.2)
        print("No such nxflag")
    #print(mydat)
    #fwn="results1_add_pcc.csv"
    fw = open(fwn, 'w')
    fw.write("#method,Top100,Top200,Top300,Top400,Top500,Top600,")
    fw.write("Top1%,Top5%,Top10%,Top15%,Top20%,Top25%,")
    fw.write("tp,fn,fp,tn,p,n,ACC,Sn,Sp,MCC,Precision,F_measure,ppv,npv\n")

    ### output each features' prediction
    #fw.write("#\n")
    xrow, xcol = XX.shape
    myorder = [i for i in range(xcol)]

    for k in range(len(myorder)):
        newXX = XX[:, myorder[k]]
        if len(newXX) > 0:
            pvalues = newXX  #get_pvalues(predicts,1) # one rank
            YY = np.array(YY, dtype=int)
            #print(predicts)
            out_ranks = my_rank(pvalues, YY,
                                maxFlag)  #top100/200/.../600/top1%...
            if maxFlag:
                scale_pvalues = (pvalues - min(pvalues)) / (
                    max(pvalues) - min(pvalues))
            else:
                scale_pvalues = (pvalues - max(pvalues)) / (
                    min(pvalues) - max(pvalues))
            #confuse matrix
            #confuse=evaluate_predictions(scale_pvalues,YY,wfnFlag)
            confuse = evaluate_predictions_4nx(scale_pvalues, YY, wfnFlag,
                                               Pnum, maxFlag)
            #write the results for each features
            for i in range(len(out_ranks)):
                if i == 0:
                    fw.write("%s,%d," % (header[myorder[k]], out_ranks[i][1]))
                else:
                    fw.write("%d," % out_ranks[i][1])

            for i in range(len(confuse)):
                fw.write("%f," % confuse[i])
            fw.write('\n')
    #### output min/max/average all features' rediction
    fw.write("#\n")  #just empty line
    #maxFlag=False
    #wfnFlag=False
    newXX = XX[:, myorder[0:-1]]
    YY = np.array(YY, dtype=int)

    pvalues = np.min(newXX, axis=1)
    out_ranks = my_rank(pvalues, YY, maxFlag)
    if maxFlag:
        scale_pvalues = (pvalues - min(pvalues)) / (
            max(pvalues) - min(pvalues))
    else:
        scale_pvalues = (pvalues - max(pvalues)) / (
            min(pvalues) - max(pvalues))
    #confuse=evaluate_predictions(scale_pvalues,YY,wfnFlag)
    confuse = evaluate_predictions_4nx(scale_pvalues, YY, wfnFlag, Pnum,
                                       maxFlag)
    for i in range(len(out_ranks)):
        if i == 0:
            fw.write("min,%d," % out_ranks[i][1])
        else:
            fw.write("%d," % out_ranks[i][1])
    for i in range(len(confuse)):
        fw.write("%f," % confuse[i])
    fw.write('\n')
    #### max
    pvalues = np.max(newXX, axis=1)
    out_ranks = my_rank(pvalues, YY, maxFlag)
    if maxFlag:
        scale_pvalues = (pvalues - min(pvalues)) / (
            max(pvalues) - min(pvalues))
    else:
        scale_pvalues = (pvalues - max(pvalues)) / (
            min(pvalues) - max(pvalues))
    #confuse=evaluate_predictions(scale_pvalues,YY,wfnFlag)
    confuse = evaluate_predictions_4nx(scale_pvalues, YY, wfnFlag, Pnum,
                                       maxFlag)

    for i in range(len(out_ranks)):
        if i == 0:
            fw.write("max,%d," % out_ranks[i][1])
        else:
            fw.write("%d," % out_ranks[i][1])
    for i in range(len(confuse)):
        fw.write("%f," % confuse[i])
    fw.write('\n')
    ### average

    np.savetxt("check.xx.order.csv", newXX, delimiter=",", fmt="%f")
    pvalues = np.average(newXX, axis=1)
    out_ranks = my_rank(pvalues, YY, maxFlag)
    if maxFlag:
        scale_pvalues = (pvalues - min(pvalues)) / (
            max(pvalues) - min(pvalues))
    else:
        scale_pvalues = (pvalues - max(pvalues)) / (
            min(pvalues) - max(pvalues))

    #confuse=evaluate_predictions(scale_pvalues,YY,wfnFlag)
    confuse = evaluate_predictions_4nx(scale_pvalues, YY, wfnFlag, Pnum,
                                       maxFlag)

    for i in range(len(out_ranks)):
        if i == 0:
            fw.write("average,%d," % out_ranks[i][1])
        else:
            fw.write("%d," % out_ranks[i][1])
    for i in range(len(confuse)):
        fw.write("%f," % confuse[i])
    fw.write('\n')
    ### median
    #wfnFlag=False #True
    #np.savetxt("check.xx.order.csv",newXX,delimiter=",",fmt="%f")
    pvalues = np.median(newXX, axis=1)
    out_ranks = my_rank(pvalues, YY, maxFlag)
    if maxFlag:
        scale_pvalues = (pvalues - min(pvalues)) / (
            max(pvalues) - min(pvalues))
    else:
        scale_pvalues = (pvalues - max(pvalues)) / (
            min(pvalues) - max(pvalues))

    #confuse=evaluate_predictions(scale_pvalues,YY,wfnFlag)
    confuse = evaluate_predictions_4nx(scale_pvalues, YY, wfnFlag, Pnum,
                                       maxFlag)

    for i in range(len(out_ranks)):
        if i == 0:
            fw.write("median,%d," % out_ranks[i][1])
        else:
            fw.write("%d," % out_ranks[i][1])
    for i in range(len(confuse)):
        fw.write("%f," % confuse[i])
    fw.write('\n')

    fw.close()
    return (0)


"""            
"""


def order_statis(XX):
    ### order get rank ratio
    XX = np.array(XX)
    #r1=len(XX)
    #c1=len(list(XX[0]))
    r1, c1 = XX.shape

    ranks = []
    for j in range(c1):
        onecol = XX[:, j]
        sort_idxes = np.argsort(-onecol)  #decrease
        onecol_rank = np.zeros((r1, 1))  ## store ranks
        #prevalue=onecol[sort_idxes[0]]
        for k in range(len(sort_idxes)):
            idx = sort_idxes[k]
            if k == 0:
                prevalue = onecol[idx]
                prerank = k + 1
                onecol_rank[idx] = k + 1
            if k == len(sort_idxes) - 1:
                onecol_rank[idx] = r1  #last rank is number of row
            else:  ###
                curvalue = onecol[idx]
                if np.allclose(curvalue, prevalue):  #same value use same rank
                    onecol_rank[idx] = prerank
                else:
                    onecol_rank[idx] = k + 1
                    prerank = k + 1

                prevalue = onecol[idx]
        if j == 0:
            ranks = onecol_rank
        else:
            ranks = np.column_stack((ranks, onecol_rank))
    ### chang ranks to ration
    ranksRatio = ranks * 1.0 / np.max(ranks, axis=0)
    #ranksRatio=ranks
    #print(ranks)
    np.savetxt("../check.myranks1.csv", ranks, fmt="%f", delimiter=",")
    return (ranksRatio)


#XX=np.array([[0.9,0.9,0.9,0.1,0.9,0.9,0.9],
#             [0.8,0.7,0.8,0.7,0.9,0.8,0.7],
#            [0.6,0.7,0.5,0.9,0.6,0.7,0.5],
#            [0.3,0.3,0.3,0.3,0.3,0.9,0.9]])
#print(XX)
#order_statis(XX)


#def main2(YY,mydat):
def get_protein_ranks(frn, fwn):
    ### get final rank ratio from features
    #frn="model.rfnwk.scale.csv"
    mydat, header, rownames = my_read_dat(frn)
    mydat = np.array(mydat)
    #YY=[int(item) for item in mydat[:,0]]
    XX = mydat[:, 1:]  # features
    YY = mydat[:, 0]  #labels
    ZZ = order_statis(XX)
    fw = open(fwn, 'w')
    ### header
    newheader = "#name,label," + ','.join(header)
    fw.write("%s\n" % newheader)
    r1, c1 = ZZ.shape
    for i in range(r1):
        for j in range(c1):
            if j == 0:
                fw.write("%s,%d," % (rownames[i], YY[i]))
                fw.write("%f," % ZZ[i][j])
            elif j == c1 - 1:
                fw.write("%f\n" % ZZ[i][j])
            else:
                fw.write("%f," % ZZ[i][j])
    fw.close()
    #np.savetxt("myranks1.csv",ZZ,fmt="%f",delimiter=",")
    return 0


#main2() #get rank ratio from features
### 12 features
#main()
def my_topn_areaRatio(scores, label, maxFlag):
    ### output top100/200/.../600; top1%,5%,...25%.
    ### results
    scores = np.array(scores, dtype=float)
    label = np.array(label, dtype=int)
    results = []
    if maxFlag:
        sort_idxes = np.argsort(-scores)  #decrease
    else:
        sort_idxes = np.argsort(scores)  #increase , less better
    #### top
    top1s = [100, 200, 300, 400, 500, 600]
    top2s = [0.01, 0.05, 0.10, 0.15, 0.20, 0.25]
    #### print top-n
    for item in top1s:
        top_idxes = sort_idxes[0:item]
        top_label = list(label[top_idxes])
        mycount = top_label.count(1)
        #print("Top %3d: %3d" %(item,mycount))
        #print(top_label)
        results.append((item, mycount))
    #### print top-%
    #print("##### percent")
    for item in top2s:
        nn = int(np.ceil(item * len(label)))  # ceiling
        #print(type(nn))
        top_idxes = sort_idxes[0:nn]
        top_label = list(label[top_idxes])
        mycount = top_label.count(1)
        #print("Top %.2f %4d %3d" %(item,nn,mycount))
        results.append((nn, mycount))
    topn = []
    for k in range(len(top1s)):
        topn.append(results[k][1])  #results[0:len(top1s)]
    #print(topn)
    # 2*y1+...+2*y5+y6 /  (3600)
    qscore = (sum(topn) + sum(topn[0:-1])) * 1.0 / 3600  # area ration
    return ((results, qscore))


if False:
    xfwn = "../myranks4pros.csv"
    #get_protein_ranks(xfrn,xfwn)
    ### out measures
    PvalFlag = False  #True
    if PvalFlag:
        x2fwn = "../xx600results_pcc_order_pvalues.csv"
    else:
        x2fwn = "../xx600results_pcc_order_Qstatis.csv"

    #select_features(xfwn,x2fwn,PvalFlag)