# -*- coding: utf-8 -*-
"""
Created on Thu Apr 27 06:26:24 2017

@author: Cliff
"""

#coding:utf-8
'''
Author       :  ZZP
Mail         :  zhangzhaopeng@mail.nankai.edu.cn
Created Time :  2016-07-25 11:27:31

Description  :  find more essential proteins according to ecc, pcc ...
'''

import networkx as nx
import numpy as np
from scipy import stats


######gene-data
def get_gene_db(G, gene_file):
    ### get the correlation from co-expression gene
    pcc_m = {}  #gene_db
    for line in open(gene_file):
        tmp = line.strip().split()  ### genen protein  dat1 dat2....
        protein = tmp[1]
        values = [float(i) for i in tmp[2:]]
        if (protein in G.nodes()):
            ### for each node
            if (protein not in pcc_m.keys()):
                pcc_m[protein] = [values]
            #多个基因对应一个蛋白
            else:
                pcc_m[protein].append(values)
    ## gene database
    return (pcc_m)


def GENE_CORR(geneDB, u, v, PCCFlag=True):
    ### compute pcc
    max_corr = 0.
    pcc_m = geneDB
    if (u in pcc_m and v in pcc_m):
        for array1 in pcc_m[u]:
            for array2 in pcc_m[v]:
                if PCCFlag:
                    corr = abs(np.corrcoef([array1,
                                            array2])[0, 1])  #2*2 matrix
                else:
                    corr = abs(stats.spearmanr(array1, array2)[0])
                if (corr >= max_corr):
                    ### maximual of one protein
                    max_corr = corr
    else:
        max_corr = 0.0
        #print "no gen"
    return max_corr


###### structure
def get_stru_db(G, structure_file):
    #### structure file
    ### 1-20 amino acid composition; 3 CHE, 1RSA;
    ### 2 cos/sin_phi 2 cos/sin_psi; 2 cos/sin theta 2 cos/sin tau
    ### 3 cna hsaup hsadw; 3 cnb hsbup hsbdw.
    ### structure database
    str_m = {}
    for line in open(structure_file):
        tmp = line.strip().split()  # protein structure-features
        protein = tmp[0]
        values = [float(i) for i in tmp[1:]]
        if (protein in G.nodes()):
            if (protein not in str_m.keys()):
                str_m[protein] = [values]
            else:
                str_m[protein].append(values)
    ### structure dabase
    return (str_m)


def STRU_CORRS(struDB, u, v, PCCFlag=True):
    ### get the structure features correlation
    ### 1-20 amino acid composition; 3 CHE, 1RSA;
    ### 2 cos/sin_phi 2 cos/sin_psi; 2 cos/sin theta 2 cos/sin tau
    ### 3 cna hsaup hsadw; 3 cnb hsbup hsbdw.

    max_corr = [0.0, 0.0, 0.0, 0.0]
    str_m = struDB
    if (u in str_m and v in str_m):
        for array1 in str_m[u]:
            for array2 in str_m[v]:
                if PCCFlag:
                    ### 20 amino acid
                    c0 = abs(np.corrcoef(array1[:20], array2[:20])[0, 1])
                    ### CHE/ RSA 4
                    c1 = abs(np.corrcoef(array1[20:24],
                                         array2[20:24])[0, 1])  #4 CHE RSA
                    ### cos/sin_phi/psi/theta/tau
                    c2 = abs(np.corrcoef(array1[24:32], array2[24:32])[0, 1])
                    ### cna/hsaup/hsadw/cnb/hsbup/hsbdw
                    c3 = abs(np.corrcoef(array1[32:38], array2[32:38])[0, 1])
                else:
                    ### 20 amino acid
                    c0 = abs(stats.spearmanr(array1[:20], array2[:20])[0])
                    #print(u,v)
                    #print(array1[:20])
                    #print(array2[:20])
                    #print(c0)
                    ### CHE/ RSA 4
                    c1 = abs(stats.spearmanr(array1[20:24],
                                             array2[20:24])[0])  #4 CHE RSA
                    ### cos/sin_phi/psi/theta/tau
                    c2 = abs(stats.spearmanr(array1[24:32], array2[24:32])[0])
                    ### cna/hsaup/hsadw/cnb/hsbup/hsbdw
                    c3 = abs(stats.spearmanr(array1[32:38], array2[32:38])[0])

                #####
                if (c0 >= max_corr[0]):
                    max_corr[0] = c0
                if (c1 >= max_corr[1]):
                    max_corr[1] = c1
                if (c2 >= max_corr[2]):
                    max_corr[2] = c2
                if (c3 >= max_corr[3]):
                    max_corr[3] = c3
    else:
        #print "no str"
        max_corr = [0.0, 0.0, 0.0, 0.0]
    return max_corr


#get_stru_db(G,structure_file)
####  localtion
def get_subloc_db(sublocf):
    ### sublocf
    fr = open(sublocf, 'r')
    lines = [line.strip() for line in fr.readlines()]
    fr.close()
    ###
    prolcDB = {}  # ProteinID: local(11-D vector)
    sublocDB = {
        'Cytoskeleton': 0,
        'Cytosol': 1,
        'Endosome': 2,
        'Extracellular': 3,
        'Endoplasmic': 4,
        'Golgi': 5,
        'Mitochondrion': 6,
        'Nucleus': 7,
        'Peroxisome': 8,
        'Plasma': 9,
        'Vacuole': 10
    }
    for line in lines:
        if line:
            aline = line.split()
            proid = aline[0]  #protein id
            lcinf = aline[1]  # location
            if len(aline) >= 3:
                if aline[1] == "Extracellular" and aline[2] == "region":
                    lcinf = "Extracelluar"
                else:
                    lcinf = "X"
                if aline[1] == "Endoplasmic" and aline[2] == "reticulum":
                    lcinf = "Endoplasmic"
                else:
                    lcinf = "X"
                if aline[1] == "Golgi" and aline[2] == "apparatus":
                    lcinf = "Golgi"
                    ##only keep Golgi apparatus/ remove Golgi membrane
                else:
                    lcinf = "X"
                if aline[1] == "Plasma" and aline[2] == "membrane":
                    lcinf = "Plasma"
                else:
                    lcinf = "X"

            if proid not in prolcDB:
                prolcDB[proid] = [0
                                  for k in range(11)]  #11 subcelluar location
                ### find the localtion index
                try:
                    idx = sublocDB[lcinf]
                except KeyError:
                    idx = -1
                if idx >= 0:
                    prolcDB[proid][idx] = 1
            else:
                try:
                    idx = sublocDB[lcinf]
                except KeyError:
                    idx = -1
                if idx >= 0:
                    prolcDB[proid][idx] = 1


#    print(prolcDB['YPR204C-A']) ### key protein
    return (prolcDB)


def SUBLOC_CORR(prolcDB, u, v):

    #### get the location edge
    ### [l1,l2,...,l11] [m1,m2,...,m11] if li==mi; return 1;
    edgeFlag = 0

    if u in prolcDB:
        ulst = prolcDB[u]
        #print(ulst)
        if v in prolcDB:
            vlst = prolcDB[v]
            #print(vlst)
            for k in range(len(ulst)):
                if ulst[k] == 1 and vlst[k] == 1:
                    edgeFlag = 1
                    #print(k)
                    break
    return (edgeFlag)


#sublocf="./2016-SON/subcellular.txt"
#prolcDB=get_subloc_db(sublocf)
#print(SUBLOC_CORR(prolcDB,'Q0045','YPR204C-A'))
import copy


def refine_network_subloc(xG, sublocf):
    ### refine network G by subcellular location
    #sublocf="../2016_SON/subcellular.txt"
    prolcDB = get_subloc_db(sublocf)
    G = copy.deepcopy(xG)
    remove_edges = []
    for u, v in G.edges():
        if SUBLOC_CORR(prolcDB, u, v):
            pass
        else:
            remove_edges.append((u, v))
    G.remove_edges_from(remove_edges)
    #### refine network
    return (G)


#### ECC
def ECC(G, u, v):
    ### edge clustering coefficient (ECC)
    num_join = 0
    for i in G.neighbors(u):
        if (i in G.neighbors(v)):
            num_join += 1
    ecc = 0.
    dc1 = G.degree(u)  #len(G.neighbors(u)) #degree u
    dc2 = G.degree(v)  #len(G.neighbors(v))  #degree v
    if dc1 - 1 == 0 or dc2 - 1 == 0:
        ecc = 0.0
    else:
        ecc = (num_join * 1.0) / min(dc1 - 1, dc2 - 1)
    return (ecc)


def get_NCDB(G):
    ### add weighted of ecc
    NCDB = {}
    for u in G.nodes():
        if u not in NCDB:
            NCDB[u] = 0
            for v in G.neighbors(u):
                NCDB[u] += ECC(G, u, v)

    ### Key: protein value NC
    return (NCDB)


def get_ECCGENE_DB(G, geneDB, PCCFlag=True, WFlag=False):
    ### update ecc with gene
    ecc_gene_db = {}
    for u in G.nodes():
        if u not in ecc_gene_db:
            ecc_gene_db[u] = 0
            for v in G.neighbors(u):
                if WFlag:  #weighted
                    ecc_gene_db[u] += (ECC(G, u, v) * 0.5 +
                                       GENE_CORR(geneDB, u, v, PCCFlag) * 0.5)
                else:
                    ecc_gene_db[u] += ECC(G, u, v) * GENE_CORR(
                        geneDB, u, v, PCCFlag)

    return (ecc_gene_db)


def get_ECCSTRU_DB(G, struDB, PCCFlag=True, WFlag=False):
    #STRU_CORRS(struDB,u,v)
    ### update ecc with structure correlation
    ecc_gene_db = {}
    for u in G.nodes():
        if u not in ecc_gene_db:
            ecc_gene_db[u] = [0, 0, 0, 0]  #4 structure index
            for v in G.neighbors(u):
                stru_lst = STRU_CORRS(struDB, u, v, PCCFlag)
                for k in range(4):
                    if WFlag:
                        ecc_gene_db[u][k] += (
                            ECC(G, u, v) * 0.5 + stru_lst[k] * 0.5)
                    else:
                        ecc_gene_db[u][k] += ECC(G, u, v) * stru_lst[k]
    ##key proteins values [l1,l2,l3,l4]
    return (ecc_gene_db)


#### gene datat to refine
def get_gene_activeDB(gene_file):
    ### get the correlation from co-expression gene
    #genedb={} #gene_db gene: pro
    prodb = {}  # pro_db   pro: gene-expression
    for line in open(gene_file):
        tmp = line.strip().split()  ### genen protein  dat1 dat2....
        #gene=tmp[0]
        protein = tmp[1]
        values = [float(i) for i in tmp[2:]]
        if True:
            ### for each node
            if (protein not in prodb.keys()):
                prodb[protein] = [values]
            #多个基因对应一个蛋白
            else:
                prodb[protein].append(values)
    ## gene database: protein: [gene1 36values],[gene2 36 values],....
    KK = 1  #2.5
    N = 36  #36 times
    probActiveDB = {}  # protein:10000..1,  (36 0/1)
    for apro in prodb.keys():
        xvalues = prodb[apro]
        a_ativ = [0 for k in range(N)]
        for k in range(len(xvalues)):
            arow = xvalues[k]
            my_mu = np.average(arow)
            my_sd = np.std(arow, ddof=1)
            my_f = 1.0 / (1.0 + my_sd**2)
            my_cutoff = my_mu + KK * my_sd * (1 - my_f)
            #print(apro,my_mu,my_sd,my_f,my_cutoff)
            for j in range(len(arow)):
                if arow[j] > my_cutoff:
                    a_ativ[j] = 1
        probActiveDB[apro] = a_ativ
    ### protein: [1 0 0 0 0 ... 1]
    return (probActiveDB)


def gene_co_active(proActiveDB, u, v):
    ### u,v two proteins  [0,1,1,,]
    activeFlag = False
    if u in proActiveDB:
        ulst = proActiveDB[u]
        #print(ulst)
        if v in proActiveDB:
            vlst = proActiveDB[v]
            #print(vlst)
            for k in range(len(ulst)):
                if ulst[k] * vlst[k] == 1:
                    activeFlag = True

    return (activeFlag)


#gene_file="../dat/eg.genedata.txt"
#aa=get_gene_activeDB(gene_file)
#print(aa)
#print(gene_co_active(aa,'YNR066C','YGR062C'))
def refine_network_gene(xG, gene_file):
    ### refine network G by subcellular location
    #gene_file="../dat/genedata.txt"
    proActiveDB = get_gene_activeDB(gene_file)
    G = copy.deepcopy(xG)
    remove_edges = []
    for u, v in G.edges():
        if gene_co_active(proActiveDB, u, v):
            pass
        else:
            remove_edges.append((u, v))
    G.remove_edges_from(remove_edges)
    #### refine network
    return (G)


from api_blast import get_evalues_db


def get_ECCblast_DB(G, evalueDB, transFlag=True, WFlag=False):
    #STRU_CORRS(struDB,u,v)
    ### update ecc with structure correlation
    evalueDB = get_evalues_db(transFlag)
    ecc_gene_db = {}
    for u in G.nodes():
        for v in G.neighbors(u):
            try:
                myweight = evalueDB[u][v]
            except KeyError:
                myweight = 0.0
            if WFlag:
                ecc_gene_db[u] += (ECC(G, u, v) * 0.5 + myweight * 0.5)
            else:
                ecc_gene_db[u] += (ECC(G, u, v) * myweight)


##key proteins values [l1,l2,l3,l4]
    return (ecc_gene_db)
